@@ -28,7 +28,8 @@ import sys
 # Local modules
 from .config import Var, log_message, create_dirs, VERSION
 from .filters import subscribe, subscribe_custom, update_all, change_permissions, unsubscribe, rename_custom, \
-    sync_filter_list_repo, show_info, FilterPermissions, list_filter_lists, search_filter_lists, purge_cache
+    sync_filter_list_repo, show_info, FilterPermissions, list_filter_lists, search_filter_lists, purge_cache, \
+    query_domain_to_filter_list
 from .hosts import update_hosts, restore_hosts, gen_hosts
 from .style import show_status
 from .daemon import start_daemon
@@ -248,6 +249,13 @@ def __setup_args_tblock(args: list) -> Arguments:
         description="Perform a search inside filter lists database",
         group="Search"
     )
+    parser.add_operation(
+        operation="--which",
+        short_operation="-W",
+        value_type=list,
+        description="Find which filter list is managing specified domain(s)",
+        group="Search"
+    )
 
     # Add "General" options
     parser.add_option(
@@ -269,6 +277,14 @@ def __setup_args_tblock(args: list) -> Arguments:
         available_with_operations=["--redirect"],
         required_by_operations=["--redirect"]
     )
+    parser.add_option(
+        option="--rebuild-hosts",
+        short_option="-o",
+        value_type=bool,
+        description="Re-build the whole hosts file after saving rules",
+        group="Rules",
+        available_with_groups=["Rules"]
+    )
 
     # Add "Filter lists" options
     parser.add_option(
@@ -460,6 +476,13 @@ def __setup_args_tblockc(args: list) -> Arguments:
         description="Redirect domains to 0.0.0.0 instead of 127.0.0.1 (hosts/dnsmasq)",
         available_with_operations=["--convert"]
     )
+    parser.add_option(
+        option="--optimize",
+        short_option="-z",
+        value_type=bool,
+        description="Do not convert blank lines",
+        available_with_operations=["--convert"]
+    )
     parser.add_option(
         option="--server",
         short_option="-e",
@@ -544,25 +567,29 @@ def __parse_args_tblock(args: list) -> None:
         allow_domains(
             domains=operations["--allow"],
             force=options["--force"],
-            quiet=False
+            quiet=False,
+            also_update_hosts=options["--rebuild-hosts"]
         )
     elif operations["--block"]:
         block_domains(
             domains=operations["--block"],
             force=options["--force"],
-            quiet=False
+            quiet=False,
+            also_update_hosts=options["--rebuild-hosts"]
         )
     elif operations["--redirect"]:
         redirect_domains(
             domains=operations["--redirect"],
             ip=options["--ip"],
             force=options["--force"],
-            quiet=False
+            quiet=False,
+            also_update_hosts=options["--rebuild-hosts"]
         )
     elif operations["--delete-rule"]:
         delete_rules(
             domains=operations["--delete-rule"],
-            quiet=False
+            quiet=False,
+            also_update_hosts=options["--rebuild-hosts"]
         )
     elif operations["--sync"]:
         sync_filter_list_repo(
@@ -666,6 +693,8 @@ def __parse_args_tblock(args: list) -> None:
             subscribing_only=options["--subscribing"],
             not_subscribing_only=options["--not-subscribing"]
         )
+    elif operations["--which"]:
+        query_domain_to_filter_list(domains=operations["--which"])
 
 
 def __parse_args_tblockc(args: list) -> None:
@@ -695,7 +724,8 @@ def __parse_args_tblockc(args: list) -> None:
             input_syntax=options["--input-syntax"],
             allow_comments=options["--comments"],
             redirect_to_zero=options["--zero"],
-            dnsmasq_server=options["--server"]
+            dnsmasq_server=options["--server"],
+            optimize=options["--optimize"]
         )
 
 
@@ -737,7 +767,7 @@ def run(args: list = None) -> None:
             log_message(f"[core]    ERROR: database operation failed with error: {0}".format(err.__str__()))
     try:
         __parse_args_tblock(args)
-    except DatabaseLockedError as err:
+    except (DatabaseLockedError, sqlite3.OperationalError) as err:
         print(Fore.YELLOW + "Error: {0}".format(err.__str__()) + Style.RESET_ALL)
         raise SystemExit(1)
     except (RuleError, TBlockNetworkError, HostsError, FilterError,
