@@ -23,7 +23,6 @@
 
 # Standard modules
 import io
-import re
 import sqlite3
 
 # Local modules
@@ -32,6 +31,7 @@ from ..style import loading_icon, Icon, Font
 from ..const import USER_RULE_PRIORITY, RulePolicy, TBlockSyntaxStatus, FilterSyntax, FilterPermissions, RULE_COMMENTS
 from ..utils import contains_wildcards
 from ..exceptions import FilterSyntaxError
+from ..config import Path
 
 
 ALL_SYNTAX = [FilterSyntax.ADBLOCKPLUS, FilterSyntax.DNSMASQ, FilterSyntax.HOSTS, FilterSyntax.LIST,
@@ -70,8 +70,7 @@ class FilterParser:
             else:
                 self.syntax = syntax
 
-    def insert_rules_to_database(self, connection: sqlite3.Connection, filter_id: str,
-                                 permissions: FilterPermissions) -> None:
+    def insert_rules_to_database(self, filter_id: str, permissions: FilterPermissions) -> None:
         try:
             with io.open(self.file, "rt") as f:
                 data = f.readlines()
@@ -85,10 +84,9 @@ class FilterParser:
         percent = 0
         all_rules_count = len(data)
 
+        connection = sqlite3.connect(Path.DATABASE)
         cursor = connection.cursor()
 
-        wildcards = cursor.execute("SELECT domain FROM rules WHERE domain LIKE '%*%';").fetchall()
-
         # TBlock syntax support
         tblock_begin = False
         tblock_policy = None
@@ -137,29 +135,21 @@ class FilterParser:
                     query = cursor.execute("SELECT domain, filter_id, policy FROM rules WHERE domain=?",
                                            (rule[0],)).fetchall()
 
-                    # Check whether a wildcard rule already exists in the database
-
-                    wildcards_exists = False
-                    for rule in wildcards:
-                        if re.match(re.compile(rule[0].replace(".", "\\.").replace("*", ".*")), domain):
-                            wildcards_exists = True
-
-                    if not wildcards_exists:
-                        # Check if the rule exists and if the rule is not a user rule
-                        if query and query[0][1] != USER_RULE_PRIORITY:
-                            # Check if the local rule has a higher priority (block < redirect < allow)
-                            if query[0][2] == policy == RulePolicy.BLOCK or \
-                                    policy == RulePolicy.REDIRECT and policy != RulePolicy.ALLOW or \
-                                    policy == RulePolicy.ALLOW:
-                                cursor.execute(
-                                    "UPDATE rules SET policy=?, filter_id=?, ip=? WHERE domain=?;",
-                                    (policy, filter_id, ip, domain)
-                                )
-                        elif not query:
+                    # Check if the rule exists and if the rule is not a user rule
+                    if query and query[0][1] != USER_RULE_PRIORITY:
+                        # Check if the local rule has a higher priority (block < redirect < allow)
+                        if query[0][2] == policy == RulePolicy.BLOCK or \
+                                policy == RulePolicy.REDIRECT and policy != RulePolicy.ALLOW or \
+                                policy == RulePolicy.ALLOW:
                             cursor.execute(
-                                "INSERT INTO rules (domain, policy, filter_id, ip) VALUES (?, ?, ?, ?);",
-                                (domain, policy, filter_id, ip)
+                                "UPDATE rules SET policy=?, filter_id=?, ip=? WHERE domain=?;",
+                                (policy, filter_id, ip, domain)
                             )
+                    elif not query:
+                        cursor.execute(
+                            "INSERT INTO rules (domain, policy, filter_id, ip) VALUES (?, ?, ?, ?);",
+                            (domain, policy, filter_id, ip)
+                        )
 
         if not self.quiet:
             print(" {0} Inserting rules into database ({1}): {2}%".format(Icon.SUCCESS, all_rules_count, percent))
@@ -168,8 +158,11 @@ class FilterParser:
                 str(round(((len(data) - connection.total_changes) * 100 / len(data)), 1))
             ))
 
+        connection.commit()
+        connection.close()
+
     def convert(self, output_file: str, output_syntax: str, allow_comments: bool = False,
-                redirect_to_zero: bool = False, dnsmasq_server: bool = False) -> None:
+                redirect_to_zero: bool = False, dnsmasq_server: bool = False, optimize: bool = False) -> None:
         try:
             with io.open(self.file, "rt") as f:
                 data = f.readlines()
@@ -290,6 +283,8 @@ class FilterParser:
                             raise FilterSyntaxError(
                                 "invalid output syntax: {0}".format(output_syntax)
                             )
+                elif line == "\n" and not optimize:
+                    w.write("\n")
 
             if output_syntax == FilterSyntax.TBLOCK:
                 w.write("\n\n@END_RULES")
@@ -302,12 +297,12 @@ class FilterParser:
 
 def convert(input_file: str, output_file: str, output_syntax: str, input_syntax: str = None,
             allow_comments: bool = False, redirect_to_zero: bool = False, dnsmasq_server: bool = False,
-            quiet: bool = False) -> None:
+            optimize: bool = False, quiet: bool = False) -> None:
     if not quiet:
         print(f"{Font.BOLD}==> Converting filter list{Font.DEFAULT}")
     filter_parser = FilterParser(input_file, syntax=input_syntax, quiet=quiet)
     filter_parser.convert(output_file, output_syntax, allow_comments=allow_comments,
-                          redirect_to_zero=redirect_to_zero, dnsmasq_server=dnsmasq_server)
+                          redirect_to_zero=redirect_to_zero, dnsmasq_server=dnsmasq_server, optimize=optimize)
 
 
 def detect_filter_list_syntax(input_file: str) -> None:
