@@ -32,13 +32,14 @@ import lzma
 import gzip
 
 # Local modules
-from .const import FilterPermissions, WARNS
+from .const import FilterPermissions, WARNS, RulePolicy
 from .style import Icon, Font, loading_icon
 from .config import Path, Var
 from .utils import fetch_file, prompt_user, is_url, check_root_access, lock_db, unlock_db
 from .exceptions import FilterError, RepoError, FilterSyntaxError
 from .converter import detect_syntax, FilterParser, ALL_SYNTAX
 from .hosts import update_hosts
+from .rules import Rule, __remove_allowed_matches
 # To avoid confusion when updating the filter list repository
 from .config import VERSION as TBLOCK_VERSION
 
@@ -62,12 +63,12 @@ class Filter:
                 "SELECT source, metadata, subscribing, on_repo, permissions, mirrors, syntax, rules_count "
                 "FROM filters WHERE id=?;",
                 (self.id,)
-            ).fetchall()
+            ).fetchone()
 
         self.exists = bool(data)
 
         if self.exists:
-            self.source = data[0][0]
+            self.source = data[0]
         elif custom_source and not self.exists:
             self.source = custom_source
         else:
@@ -78,19 +79,19 @@ class Filter:
             check = conn.cursor().execute(
                 "SELECT id FROM filters WHERE source=?;",
                 (self.source,)
-            ).fetchall()
+            ).fetchone()
         if check:
             self.source_exists = True
         else:
             self.source_exists = False
 
-        self.metadata = json.loads(data[0][1]) if self.exists else {}
-        self.subscribing = bool(data[0][2]) if self.exists else False
-        self.on_repo = bool(data[0][3]) if self.exists else False
-        self.permissions = FilterPermissions(data[0][4]) if self.exists else None
-        self.mirrors = json.loads(data[0][5]) if (self.exists and data[0][5]) else {}
-        self.syntax = data[0][6] if self.exists else None
-        self.rules_count = data[0][7] if self.exists else None
+        self.metadata = json.loads(data[1]) if self.exists else {}
+        self.subscribing = bool(data[2]) if self.exists else False
+        self.on_repo = bool(data[3]) if self.exists else False
+        self.permissions = FilterPermissions(data[4]) if self.exists else None
+        self.mirrors = json.loads(data[5]) if (self.exists and data[5]) else {}
+        self.syntax = data[6] if self.exists else None
+        self.rules_count = data[7] if self.exists else None
         self.tmp_file = os.path.join(Path.TMP_DIR, self.id.replace(":", "_").replace("/", "_").replace("?", "_"))
         self.cache_file = os.path.join(Path.CACHE, self.id.replace(":", "_").replace("/", "_").replace("?", "_"))
 
@@ -245,10 +246,8 @@ class Filter:
             if not self.quiet:
                 print(f" {Icon.INFORMATION} " + "Filter list syntax is: {0}".format(self.syntax))
 
-            with sqlite3.connect(Path.DATABASE) as conn:
-                parser = FilterParser(self.tmp_file, self.syntax, self.quiet)
-                parser.insert_rules_to_database(conn, self.id, self.permissions)
-                conn.commit()
+            parser = FilterParser(self.tmp_file, self.syntax, self.quiet)
+            parser.insert_rules_to_database(self.id, self.permissions)
 
             # Move downloaded filter list to cache and insert its hash into the database
             with open(self.tmp_file, 'rb') as f:
@@ -299,7 +298,7 @@ class Filter:
                 (None, self.id)
             )
             conn.commit()
-            total_changes = conn.total_changes
+            total_changes = conn.total_changes - 1
         if not self.quiet:
             print(f' {Icon.SUCCESS} {__msg} {total_changes}')
 
@@ -326,8 +325,9 @@ class Filter:
                 self.permissions = permissions
                 self.subscribing = True
             if not self.quiet:
-                if self.metadata["warning"] > 0:
-                    print(f" {Icon.WARNING} Warning: filter list {WARNS[self.metadata['warning']]}")
+                if self.on_repo:
+                    if self.metadata["warning"] > 0:
+                        print(f" {Icon.WARNING} Warning: filter list {WARNS[self.metadata['warning']]}")
                 print(f" {Icon.SUCCESS} {__msg}")
 
     def add_custom(self, custom_syntax: str = None) -> None:
@@ -541,23 +541,23 @@ def get_all_filter_lists(subscribing_only: bool = False, not_subscribing_only: b
     else:
         with sqlite3.connect(Path.DATABASE) as conn:
             if subscribing_only and from_repo_only:
-                data = conn.execute("SELECT id FROM filters WHERE subscribing=1 AND on_repo=1;").fetchall()
+                data = conn.execute("SELECT id FROM filters WHERE subscribing=1 AND on_repo=1;")
             elif subscribing_only and custom_only:
-                data = conn.execute("SELECT id FROM filters WHERE subscribing=1 AND on_repo=0;").fetchall()
+                data = conn.execute("SELECT id FROM filters WHERE subscribing=1 AND on_repo=0;")
             elif subscribing_only:
-                data = conn.execute("SELECT id FROM filters WHERE subscribing=1;").fetchall()
+                data = conn.execute("SELECT id FROM filters WHERE subscribing=1;")
             elif not_subscribing_only and from_repo_only:
-                data = conn.execute("SELECT id FROM filters WHERE subscribing=0 AND on_repo=1;").fetchall()
+                data = conn.execute("SELECT id FROM filters WHERE subscribing=0 AND on_repo=1;")
             elif not_subscribing_only and custom_only:
-                data = conn.execute("SELECT id FROM filters WHERE subscribing=0 AND on_repo=0;").fetchall()
+                data = conn.execute("SELECT id FROM filters WHERE subscribing=0 AND on_repo=0;")
             elif not_subscribing_only:
-                data = conn.execute("SELECT id FROM filters WHERE subscribing=0;").fetchall()
+                data = conn.execute("SELECT id FROM filters WHERE subscribing=0;")
             elif custom_only:
-                data = conn.execute("SELECT id FROM filters WHERE on_repo=0;").fetchall()
+                data = conn.execute("SELECT id FROM filters WHERE on_repo=0;")
             elif from_repo_only:
-                data = conn.execute("SELECT id FROM filters WHERE on_repo=1;").fetchall()
+                data = conn.execute("SELECT id FROM filters WHERE on_repo=1;")
             else:
-                data = conn.execute("SELECT id FROM filters;").fetchall()
+                data = conn.execute("SELECT id FROM filters;")
         output_list = []
         for row in data:
             if blacklist and row[0] in blacklist:
@@ -631,6 +631,7 @@ def subscribe(filter_lists: list, do_not_prompt: bool = False, quiet: bool = Fal
                     filter_object = Filter(filter_id, quiet)
                     filter_object.subscribe(permissions=permissions)
                     filter_object.update(force=force)
+                __remove_allowed_matches(quiet)
                 update_hosts(quiet=quiet)
 
     else:
@@ -653,6 +654,7 @@ def subscribe(filter_lists: list, do_not_prompt: bool = False, quiet: bool = Fal
                     filter_object = Filter(filter_id, quiet)
                     filter_object.subscribe(permissions=permissions)
                     filter_object.update(force=force)
+                __remove_allowed_matches(quiet)
                 update_hosts(quiet=quiet)
 
     # Unlock the database
@@ -705,6 +707,7 @@ def change_permissions(filter_lists: list, permissions: FilterPermissions, do_no
                     print(f"{Font.BOLD}==> Updating filter list: {filter_id}{Font.DEFAULT}")
                 filter_object = Filter(filter_id, quiet)
                 filter_object.update(force=True)
+            __remove_allowed_matches(quiet)
             update_hosts(quiet=quiet)
 
     # Unlock the database
@@ -831,6 +834,7 @@ def subscribe_custom(filter_lists: list, do_not_prompt: bool = False, quiet: boo
                 filter_object = Filter(filter_id, quiet)
                 filter_object.subscribe(permissions=permissions)
                 filter_object.update(force=force)
+            __remove_allowed_matches(quiet)
             update_hosts(quiet=quiet)
 
     # Unlock the database
@@ -870,11 +874,13 @@ def unsubscribe(filter_lists: list, do_not_prompt: bool = False, force: bool = F
                     print(f"{Font.BOLD}==> Unsubscribing from filter list: {filter_id}{Font.DEFAULT}")
                 filter_object = Filter(filter_id, quiet)
                 filter_object.unsubscribe()
+                filter_object.delete_cache()
             if full_update:
                 update_all(do_not_prompt=True, quiet=quiet, sync_repo=False, force=force, also_update_hosts=False,
                            lock_database=False)
                 update_hosts(quiet=quiet)
             else:
+                __remove_allowed_matches(quiet)
                 update_hosts(quiet=quiet)
                 print(f" {Icon.WARNING} Always remember to update all filter lists after this operation")
 
@@ -916,6 +922,7 @@ def update(filter_lists: list, do_not_prompt: bool = False, force: bool = False,
                     print(f"{Font.BOLD}==> Updating filter list: {filter_id}{Font.DEFAULT}")
                 filter_object = Filter(filter_id, quiet)
                 filter_object.update(force=force)
+            __remove_allowed_matches(quiet)
             update_hosts(quiet=quiet)
 
     # Unlock the database
@@ -960,6 +967,7 @@ def update_all(do_not_prompt: bool = False, force: bool = False, quiet: bool = F
                     print(f"{Font.BOLD}==> Updating filter list: {filter_id}{Font.DEFAULT}")
                 filter_object = Filter(filter_id, quiet)
                 filter_object.update(force=force)
+            __remove_allowed_matches(quiet)
             if also_update_hosts:
                 update_hosts(quiet=quiet)
 
@@ -1077,6 +1085,22 @@ def purge_cache(do_not_prompt: bool = False, quiet: bool = False) -> None:
             Filter(filter_id, quiet=quiet).delete_cache()
 
 
+def query_domain_to_filter_list(domains: list, quiet: bool = False) -> None:
+    for d in domains:
+        domain_obj = Rule(d)
+        if not domain_obj.exists:
+            if not quiet:
+                print(f" {Icon.WARNING} rule does not exist: {d}")
+        else:
+            origin = domain_obj.filter_id
+            if domain_obj.policy == RulePolicy.ALLOW:
+                print(f"{origin}: ALLOW {d}")
+            elif domain_obj.policy == RulePolicy.BLOCK:
+                print(f"{origin}: BLOCK {d}")
+            elif domain_obj.policy == RulePolicy.REDIRECT:
+                print(f"{origin}: REDIRECT {d} TO {domain_obj.ip}")
+
+
 def sync_filter_list_repo(quiet: bool = False, force: bool = False, lock_database: bool = True) -> None:
     """
     Print information about a given list of filter lists
@@ -1125,7 +1149,7 @@ def sync_filter_list_repo(quiet: bool = False, force: bool = False, lock_databas
         print(f" {Icon.ERROR} {__msg}")
         raise RepoError("wrong checksum for filter list repository")
 
-    # Parse the XML file
+    # Parse the JSON file
     __msg = "Checking repository version"
     if not quiet:
         print(f" {loading_icon(1)} {__msg}", end="\r")
@@ -1260,7 +1284,7 @@ def sync_filter_list_repo(quiet: bool = False, force: bool = False, lock_databas
                     )
                 except sqlite3.IntegrityError:
                     # That means the source is already taken
-                    conflict_id = cursor.execute("SELECT id FROM filters WHERE source=?", (source,)).fetchall()[0][0]
+                    conflict_id = cursor.execute("SELECT id FROM filters WHERE source=?", (source,)).fetchone()[0]
                     print(f" {Icon.WARNING} Local filter list source conflicts with upstream: {conflict_id}")
 
             elif filter_obj.exists and not filter_obj.on_repo:
