@@ -27,6 +27,7 @@ import os
 import sqlite3
 import ipaddress
 import urllib3
+import multiprocessing
 
 # Local modules
 from .config import Path
@@ -34,7 +35,7 @@ from .style import loading_icon, Icon, Font
 from .const import USER_RULE_PRIORITY, RulePolicy
 from .exceptions import RuleError
 from .utils import is_valid_ip, prompt_user, check_root_access, is_valid_domain, lock_db, unlock_db
-from .hosts import update_hosts
+from .hosts import update_hosts, remove_from_hosts, add_to_hosts
 
 
 class Rule:
@@ -50,16 +51,16 @@ class Rule:
             data = conn.cursor().execute(
                 'SELECT "policy", "filter_id", "ip" FROM "rules" WHERE domain=?;',
                 (self.domain,)
-            ).fetchall()
+            ).fetchone()
         self.exists = bool(data)
-        self.policy = data[0][0] if data else None
-        self.filter_id = data[0][1] if data else None
-        self.ip = data[0][2] if data else None
+        self.policy = data[0] if data else None
+        self.filter_id = data[1] if data else None
+        self.ip = data[2] if data else None
         self.is_user_rule = True if self.filter_id == USER_RULE_PRIORITY else False
 
     def wildcard_exists(self) -> bool:
         with sqlite3.connect(Path.DATABASE) as conn:
-            data = conn.cursor().execute("SELECT domain FROM rules WHERE domain LIKE '%*%';").fetchall()
+            data = conn.cursor().execute("SELECT domain FROM rules WHERE domain LIKE '%*%';")
         for rule in data:
             if re.match(re.compile(rule[0].replace(".", "\\.").replace("*", ".*")), self.domain):
                 return True
@@ -129,6 +130,7 @@ class Rule:
                             print(f" {Icon.ERROR} {__msg}")
                             print(
                                 f" {Icon.WARNING} Wildcard rule already allows domain: {self.domain}")
+
                 else:
                     raise RuleError("invalid ip address: {0}".format(ip))
             else:
@@ -155,6 +157,47 @@ class Rule:
                 print(f" {Icon.SUCCESS} {__msg}")
 
 
+def __remove_allowed_matches(quiet: bool = False) -> dict:
+    if not quiet:
+        print(f"{Font.BOLD}==> Cleaning database{Font.DEFAULT}")
+    conn = sqlite3.connect(Path.DATABASE)
+    cursor = conn.cursor()
+    wildcards = cursor.execute(
+        'SELECT domain FROM rules WHERE policy=? AND domain LIKE "%*%";', (RulePolicy.ALLOW,)
+    ).fetchall()
+
+    cnt = multiprocessing.Value('i', 0)
+
+    with multiprocessing.Pool(initializer=init_globals, initargs=(cnt,)) as pool:
+        wildcards_matches = pool.map(check_wildcard_domains, wildcards)
+    if not quiet:
+        print(" {0} Checking wildcards rules: {1}/{2}".format(Icon.SUCCESS, cnt.value, len(wildcards)))
+    matches = {}
+    for row in wildcards_matches:
+        matches.update(row)
+    return matches
+
+
+def init_globals(counter):
+    global cnt
+    cnt = counter
+
+
+def check_wildcard_domains(i) -> dict:
+    matches = {}
+    cnt.value += 1
+    print(" {0} Checking wildcards rules: {1}".format(loading_icon(cnt.value), cnt.value), end="\r")
+    conn = sqlite3.connect(Path.DATABASE)
+    cursor = conn.cursor()
+    query = cursor.execute("SELECT domain, ip FROM rules WHERE domain LIKE ? AND policy != ?;",
+                           (i[0].replace("*", "%"), RulePolicy.ALLOW)).fetchall()
+    for q in query:
+        cursor.execute("DELETE FROM rules WHERE domain=?;", (q[0],))
+        matches[q[0]] = q[1]
+    conn.commit()
+    return matches
+
+
 def allow_domains(domains: list, do_not_prompt: bool = False, force: bool = False, quiet: bool = False,
                   also_update_hosts: bool = True) -> None:
     """
@@ -175,14 +218,25 @@ def allow_domains(domains: list, do_not_prompt: bool = False, force: bool = Fals
 
     # Prompt the user before continuing
     if do_not_prompt or prompt_user("You are about to allow the following domains:", domains):
+        wildcards = False
         # Add rules
+        domains_list = {}
         if not quiet:
             print(f"{Font.BOLD}==> Adding rules{Font.DEFAULT}")
         for d in domains:
             rule_object = Rule(d)
+            domains_list[d] = Rule(d).ip
             rule_object.add(RulePolicy.ALLOW, quiet=quiet)
+            if "*" in list(d):
+                wildcards = True
+        if wildcards:
+            matches = __remove_allowed_matches(quiet)
+        else:
+            matches = {}
         if also_update_hosts:
             update_hosts(quiet=quiet)
+        else:
+            remove_from_hosts({**domains_list, **matches})
 
     # Unlock the database
     unlock_db()
@@ -216,6 +270,8 @@ def block_domains(domains: list, do_not_prompt: bool = False, force: bool = Fals
             rule_object.add(RulePolicy.BLOCK, quiet=quiet)
         if also_update_hosts:
             update_hosts(quiet=quiet)
+        else:
+            add_to_hosts(domains)
 
     # Unlock the database
     unlock_db()
@@ -250,6 +306,8 @@ def redirect_domains(domains: list, ip: str, do_not_prompt: bool = False, force:
             rule_object.add(RulePolicy.REDIRECT, ip=ip, quiet=quiet)
         if also_update_hosts:
             update_hosts(quiet=quiet)
+        else:
+            add_to_hosts(domains, ip)
 
     # Unlock the database
     unlock_db()
@@ -274,14 +332,18 @@ def delete_rules(domains: list, do_not_prompt: bool = False, quiet: bool = False
 
     # Prompt the user before continuing
     if do_not_prompt or prompt_user("You are about to delete the rules of the following domains:", domains):
+        domains_list = {}
         # Delete rules
         if not quiet:
             print(f"{Font.BOLD}==> Deleting rules{Font.DEFAULT}")
         for d in domains:
             rule_object = Rule(d)
+            domains_list[d] = Rule(d).ip
             rule_object.remove(quiet=quiet)
         if also_update_hosts:
             update_hosts(quiet=quiet)
+        else:
+            remove_from_hosts(domains_list)
 
     # Unlock the database
     unlock_db()
@@ -308,21 +370,21 @@ def list_rules(from_filter_lists: list = None, user_only: bool = False, standard
                     data = conn.cursor().execute(
                         'SELECT "domain", "policy", "ip" FROM "rules" WHERE filter_id=?;',
                         (f,)
-                    ).fetchall()
+                    )
             elif standard_only:
                 data = conn.cursor().execute(
                     'SELECT "domain", "policy", "ip" FROM "rules" WHERE filter_id!=?;',
                     (USER_RULE_PRIORITY,)
-                ).fetchall()
+                )
             elif user_only:
                 data = conn.cursor().execute(
                     'SELECT "domain", "policy", "ip" FROM "rules" WHERE filter_id=?;',
                     (USER_RULE_PRIORITY,)
-                ).fetchall()
+                )
             else:
                 data = conn.cursor().execute(
                     'SELECT "domain", "policy", "ip" FROM "rules";'
-                ).fetchall()
+                )
 
         for rule in data:
             if rule[1] == RulePolicy.ALLOW:
