@@ -27,6 +27,7 @@ import hashlib
 import requests
 import urllib3
 import ipaddress
+import socket
 
 # Standard modules
 import re
@@ -143,6 +144,21 @@ def get_readable_size(bytes_length: float) -> str:
             bytes_length /= 1024.0
 
 
+def has_internet_connection(url: str) -> bool:
+    if urllib3.get_host(url)[1] == "0.0.0.0":
+        return True
+    try:
+        host = socket.gethostbyname(urllib3.get_host(url)[1])
+        if urllib3.get_host(url)[0] == "https":
+            s = socket.create_connection((host, 443), 2)
+        else:
+            s = socket.create_connection((host, 80), 2)
+        s.close()
+        return True
+    except (socket.gaierror, OSError):
+        return False
+
+
 def is_url(string: str) -> bool:
     """
     Check if a string is a valid web URL
@@ -167,52 +183,55 @@ def fetch_file(location: str, description: str, output_file: str, quiet: bool =
     }
     __msg = "Fetching {0}".format(description)
     if is_url(location):
-        # File is online and needs to be downloaded
-        try:
-            response = requests.get(location, stream=True, headers=headers)
-            if response.status_code != 200:
-                raise TBlockNetworkError("status code is not 200")
-            online_size = int(response.headers.get('content-length', 0))
-            online_size_readable = get_readable_size(online_size)
+        if has_internet_connection(location):
+            # File is online and needs to be downloaded
+            try:
+                response = requests.get(location, stream=True, headers=headers)
+                if response.status_code != 200:
+                    raise TBlockNetworkError("status code is not 200")
+                online_size = int(response.headers.get('content-length', 0))
+                online_size_readable = get_readable_size(online_size)
 
-            local_size = 0
-            count = 0
+                local_size = 0
+                count = 0
 
-            with open(output_file, 'wb') as f:
-                for data in response.iter_content(1024):
-                    count += 1
-                    local_size += 1024
-                    if online_size != 0:
-                        percent = int(local_size * 100 / online_size)
-                    else:
-                        percent = '- '
-                    if not quiet:
-                        print(f" {loading_icon(count)} {__msg} ({online_size_readable}): {percent}%",
-                              end="\r")
-                    f.write(data)
+                with open(output_file, 'wb') as f:
+                    for data in response.iter_content(1024):
+                        count += 1
+                        local_size += 1024
+                        if online_size != 0:
+                            percent = int(local_size * 100 / online_size)
+                        else:
+                            percent = '- '
+                        if not quiet:
+                            print(f" {loading_icon(count)} {__msg} ({online_size_readable}): {percent}%",
+                                  end="\r")
+                        f.write(data)
 
-            local_size += 1024
-            if not quiet:
-                print(f" {Icon.SUCCESS} {__msg} ({get_readable_size(local_size)}): 100%")
-            if sha512sum is not None:
+                local_size += 1024
                 if not quiet:
-                    print(f" {loading_icon(1)} Checking hash", end='\r')
-                with open(output_file, 'rb') as r:
-                    hash = hashlib.sha512(r.read()).hexdigest()
-                if hash == sha512sum:
+                    print(f" {Icon.SUCCESS} {__msg} ({get_readable_size(local_size)}): 100%")
+                if sha512sum is not None:
                     if not quiet:
-                        print(f" {Icon.SUCCESS} Checking hash")
-                    return True
+                        print(f" {loading_icon(1)} Checking hash", end='\r')
+                    with open(output_file, 'rb') as r:
+                        hash = hashlib.sha512(r.read()).hexdigest()
+                    if hash == sha512sum:
+                        if not quiet:
+                            print(f" {Icon.SUCCESS} Checking hash")
+                        return True
+                    else:
+                        if not quiet:
+                            print(f" {Icon.ERROR} Checking hash")
+                        return False
                 else:
-                    if not quiet:
-                        print(f" {Icon.ERROR} Checking hash")
-                    return False
-            else:
-                return True
+                    return True
 
-        except (requests.exceptions.ConnectionError, urllib3.exceptions.MaxRetryError, ConnectionRefusedError,
-                urllib3.exceptions.NewConnectionError, TBlockNetworkError):
-            print(f" {Icon.ERROR} {__msg} (0 B): 0%")
+            except (requests.exceptions.ConnectionError, urllib3.exceptions.MaxRetryError, ConnectionRefusedError,
+                    urllib3.exceptions.NewConnectionError, TBlockNetworkError):
+                print(f" {Icon.ERROR} {__msg} (0 B): 0%")
+                return False
+        else:
             return False
     else:
         try:
